/***************************************************************************
 *            client_dir_scan.c
 *
 *  Fri Oct 24 11:26:11 2003
 *  Copyright  2003  Phil Kett
 *  pkett@genesis-midi.com
 ****************************************************************************/
#include "../include/remsync.h"

int
client_dir_scan (int client_sd, char *src_dir, char *dest_dir, FILE * fd,
		 int port, char *servername)
{
	char message[BIG_BUF], command[BIG_BUF];
	char filename[BIG_BUF], csum[BIG_BUF];
	char fulldest[BIG_BUF], fullsrc[BIG_BUF], mode[BIG_BUF];
	char dummy1[BIG_BUF], dummy2[BIG_BUF];
	char owner[BIG_BUF], group[BIG_BUF];
	char mod_time[BIG_BUF], acc_time[BIG_BUF];
	int dest_len, src_mtime, dest_mtime;
	int src_len, filename_length, loop, uid, gid, atime, mtime;
	int src_crc, dest_crc, need_file = 0, isdir, result, count;
	mode_t filemode;
	struct split splitted[ARRAY_MAX_ELEM];
	struct strstat dest_stat;
	struct strstat src_stat;
	


	sprintf (command, "scan:%s\r\n", src_dir);
	send (client_sd, command, strlen (command), 0);

	umask (0000);
	while (1)
	{

		bzero (fulldest, BIG_BUF);
		bzero (fullsrc, BIG_BUF);
		bzero (filename, BIG_BUF);
		bzero (dummy1, BIG_BUF);
		bzero (dummy2, BIG_BUF);
		bzero (csum, BIG_BUF);
		bzero (owner, BIG_BUF);
		bzero (group, BIG_BUF);

		need_file = 0;
		fgets (message, BIG_BUF, fd);
		strip_string (message);
		if (strcmp (message, "DONE") == 0)
		{
			break;
		}
		if (conf.debug)
			printf ("client_dir_scan:Read %s\n", message);
		/* changed to use new split_string2 function
		 * order of strings is
		 * CSUM:curdir:mode:owner:group:mod_time:acc_time:checksum
		 */
/* 		split_string (message, csum, dummy1);
 * 		split_string (dummy1, fullsrc, dummy2);
 * 		bzero (dummy1, BIG_BUF);
 * 		split_string (dummy2, mode, dummy1);
 * 		bzero (dummy2, BIG_BUF);
 * 		split_string (dummy1, owner, dummy2);
 * 		bzero (csum, BIG_BUF);
 * 		split_string (dummy2, group, csum);
 */
		split_string2(message, splitted);
		strcpy(dummy1, splitted[0].text);
		strcpy(src_stat.filename, splitted[1].text);
		strcpy(src_stat.mode, splitted[2].text);
		strcpy(src_stat.owner, splitted[3].text);
		strcpy(src_stat.group, splitted[4].text);
		strcpy(src_stat.mod_time, splitted[5].text);
		strcpy(src_stat.acc_time, splitted[6].text);
		strcpy(csum, splitted[7].text);
		
		if (conf.debug)
		{
			printf ("client_dir_scan: fullsrc: %s\n", src_stat.filename);
			printf ("client_dir_scan: mode: %s\n", src_stat.mode);
			printf ("client_dir_scan: owner: %s\n", src_stat.owner);
			printf ("client_dir_scan: group: %s\n", src_stat.group);
			printf ("client_dir_scan: csum: %s\n", csum);
		}
		if (strcmp (src_stat.filename, src_dir) != 0)
		{
			dest_len = strlen (dest_dir);
			src_len = strlen (src_dir);
			filename_length = strlen (src_stat.filename);
			for (loop = src_len; loop < filename_length; loop++)
			{

				filename[loop - src_len] = src_stat.filename[loop];
			}
			filename[loop + 1] = '\0';
			strcat (fulldest, dest_dir);
			strcat (fulldest, filename);
		}
		else
		{
			strcat (fulldest, dest_dir);
		}
		strip_string (csum);
		filemode = atoi (src_stat.mode);
		uid = atoi (src_stat.owner);
		gid = atoi (src_stat.group);
		mtime = atoi (src_stat.mod_time);
		atime = atoi (src_stat.acc_time);
		if (strcmp (csum, "LINK") == 0)
		{
			/* this is a link -
			 * mode contains the target and not the permissions
			 */
			stats.num_links_processed += 1;
			result = symlink (src_stat.mode, fulldest);
			if (result != 0)
			{
				if (conf.debug)
					logmsg ("Symlink!");
			}
		}
		else if (strcmp (csum, "DIRECTORY") == 0)
		{
			stats.num_dirs_processed += 1;
			if (conf.debug)
				printf ("******** DIRECTORY *******\n");
			/* This is a directory report
			 * make sure it exists
			 */
			isdir = is_a_dir (fulldest);
			if (isdir == -1)
			{
				stats.num_dirs_copied += 1;
				/* Directory doesn't exist - create it */
				if (conf.debug)
					printf ("Creating directory %s\n",
						fulldest);
				if (mkdir (fulldest, filemode) != 0)
				{
					logmsg ("Unable to create dir");
					exit (-1);
				}
				else
				{
					chown (fulldest, uid, gid);
				}
			}
		}
		else
		{
			if (conf.debug)
				printf ("Dest = %s\nSrc = %s\nFilename = %s\nCsum = %s\nSrc_dir = %s\ndest_dir = %s\n", fulldest, src_stat.filename, filename, csum, src_dir, dest_dir);
			/* check that filename exists */
			stats.num_files_processed += 1;
			if (file_exists (fulldest))
			{
				dest_crc = cksum (fulldest);
				src_crc = atoi (csum);

				if (dest_crc != src_crc)
				{
					if (conf.newer) {
					/* need to check file dates here - if option is set
					 * don't copy file if dest file is newer than source
					 */
						printf("Checking file dates\n");
						result = getstat(fulldest, &dest_stat);
						src_mtime = atoi(src_stat.mod_time);
						dest_mtime = atoi(dest_stat.mod_time);
						if (src_mtime < dest_mtime) {
							/* dest is newer - don't copy */
							printf("File different but destination newer - not copying\n");
						} else {
							printf("File changed and dest not newer\n");
							need_file = 1;
						}
					
					}	else {
					
						if (conf.debug)
							printf ("File changed!\n");
						need_file = 1;
					}
				}
			}
			else
			{
				need_file = 1;
			}
		}
		if (need_file == 1)
		{
			stats.num_files_copied += 1;
			if (conf.debug)
				syslog (LOG_INFO, "Getting file %s\n",
					fullsrc);
			if (strlen (conf.notify) != 0)
			{
				notify (fulldest);
			}
			for (count = 0; count < 5; count++)
			{	/* Attempt to copy the file over a maximum of 5 times */
				result = netgetfile (src_stat.filename, fulldest, port,
						     servername, filemode,
						     uid, gid, mtime, atime);
				if (result == 0)
				{
					/* The file has copied over successfully, break the loop */
					break;
				}
				if (conf.debug)
					printf ("Retrying to get file %s\n",
						fullsrc);
			}
			/* We need the file so copy it over */
		}
	}
	return 1;
}
